/* CHAP08.C  (Chapter 8: GDI Potpourri) */

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include "chap08.h"

int PASCAL WinMain(hInstance, hPrevInst, lpszCmdLine, nCmdShow)
HANDLE hInstance;               /* current program instance */
HANDLE hPrevInst;               /* previous program instance */
LPSTR lpszCmdLine;              /* command line string */
int nCmdShow;                   /* icon or window appearance */
{
    MSG msg;

    if(!hPrevInst)
        if(!InitODcursor(hInstance))
            return(FALSE);

    if(!InitMessageBar(hInstance))
        return(FALSE);

    if(!InitClockBar(hInstance))
        return(FALSE);

     InitRectControl(hInstance);

    /* Initialize new instance of a window */

     if(!(hwndClient = CreateODwindow(hInstance, nCmdShow)))
        return(FALSE);

     hAppCursor  = LoadCursor(hInst, "AppCursor");
     hSizeCursor = LoadCursor(hInst, "SizeCursor");
     hOldCursor  = SetCursor(hAppCursor);

     if(!CreateMessageBar(hwndClient))
         return(FALSE);
     if(!CreateClockBar(hwndClient))
         return(FALSE);

     SetTimer(hwndClockBar, 1, (short)1000, NULL);
     PostMessage(hwndClockBar, WM_TIMER, 1, 0L);

     while (GetMessage(&msg, NULL, NULL, NULL))
     {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
     }
     return(msg.wParam);
}

/**********************************************************************
 *                                                                    *
 *                      Window Initializations                        *
 *                                                                    *
 **********************************************************************/

BOOL InitClockBar(hInstance)     /* FILENAME: INITCLCK.C */
HANDLE hInstance;
{
    WNDCLASS    wc;

    wc.lpszClassName = "ClockBar";
    wc.hInstance = hInstance;
    wc.lpfnWndProc = ClockBarWndProc;
    wc.hCursor = NULL;
    wc.hIcon = NULL;
    wc.lpszMenuName = NULL;
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;

    return(RegisterClass(&wc));
}

BOOL InitODcursor(hInstance)     /* FILENAME: INIODCUR.C */
HANDLE hInstance;
{
    WNDCLASS wc;

    wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    wc.lpfnWndProc = WindowProcedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "AppIcon");
    wc.hCursor = NULL;
    wc.hbrBackground = GetStockObject(GRAY_BRUSH);
    wc.lpszMenuName = (LPSTR)"MainMenu";
    wc.lpszClassName = "Ch8Class";

    return(RegisterClass(&wc));
}

BOOL InitMessageBar(hInstance)     /* FILENAME: INITMSG.C */
HANDLE hInstance;
{
    WNDCLASS wc;
    HBRUSH hBrush;
    HDC hdc;

    hdc=GetDC(GetFocus());
    hBrush=CreateSolidBrush(RGB(255, 0, 0));

    wc.lpszClassName = "MessageBar";
    wc.hInstance     = hInstance;
    wc.lpfnWndProc   = MessageBarWndProc;
    wc.hCursor       = NULL;
    wc.hIcon         = NULL;
    wc.lpszMenuName  = NULL;
    wc.hbrBackground = SelectObject(hdc, hBrush);
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;

    ReleaseDC(GetFocus(), hdc);
    DeleteObject(hBrush);
    return(RegisterClass(&wc));
}

BOOL InitRectControl(hInstance)        /* FILENAME: INITRECT.C */
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = RectWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = COLOR_WINDOW + 1;
    wc.lpszMenuName =  NULL;
    wc.lpszClassName = "Rectangle";

    return(RegisterClass(&wc));
}

HWND CreateODwindow(hInstance, nCmdShow)
HANDLE hInstance;
int nCmdShow;
{
    HWND hwnd;

    hInst = hInstance;

    hwnd = CreateWindow(
        "Ch8Class",
        "Chapter 8: GDI Potpourri",
        dwStyle,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL);

    if(!hwnd)
        return(FALSE);

    ShowWindow(hwnd,nCmdShow);
    UpdateWindow(hwnd);
    return(hwnd);
}

HWND CreateMessageBar(hwnd)     /* FILENAME: MESSGBAR.C */
HWND hwnd;
{
    int nMenu;
    RECT rc;

    GetClientRect(hwnd, &rc);
    nMenu = GetSystemMetrics(SM_CYMENU);

    hwndMsgBar = CreateWindow((LPSTR)"MessageBar",
                     (LPSTR)"",
                     WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
                     rc.left,
                     rc.bottom - nMenu,
                     rc.left + rc.right - 73,
                     nMenu,
                     hwnd,
                     IDW_STATBAR,
                     hInst,
                     NULL);

    return(hwndMsgBar);

}

HWND CreateClockBar(hwnd)        /* FILENAME: CLOCKBAR.C */
HWND hwnd;
{
    int nMenu;
    RECT rc;


    GetClientRect(hwnd, &rc);
    nMenu = GetSystemMetrics(SM_CYMENU);

    hwndClockBar = CreateWindow((LPSTR)"ClockBar",
                     (LPSTR)"",
                     WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
                     rc.right - 71,
                     rc.bottom - nMenu,
                     71,
                     nMenu,
                     hwnd,
                     IDW_CLOCKBAR,
                     hInst,
                     NULL);

    return(hwndClockBar);
}


/**********************************************************************
 *                                                                    *
 *                        Window Procedures                           *
 *                                                                    *
 **********************************************************************/

long FAR PASCAL WindowProcedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    HMENU hMenu, hFile, hEdit,
          hPup, hHelp;
    HDC hdc;
    POINT pt;
    PAINTSTRUCT ps;
    RECT rc;
    HBRUSH hBrush, hOldBrush;

    switch (msg)
    {
      case WM_CREATE:
           ScreenHeight = GetSystemMetrics(SM_CYSCREEN);
           ScreenWidth = GetSystemMetrics(SM_CXSCREEN);
           hOldCursor = SetCursor(hAppCursor);
           hMenu = GetMenu(hwnd);
           CheckMenuItem(hMenu,IDM_BITMAP, MF_UNCHECKED);
           break;

      case WM_SIZE:
           InvalidateRect(hwndClient, NULL, TRUE);
           UpdateWindow(hwndClient);
           return(TRUE);
           break;

      case WM_SETCURSOR:
           if(!IsIconic(hwnd))
             hOldCursor = SetCursor(hAppCursor);
           return(TRUE);
           break;

      case WM_NCMOUSEMOVE:
      case WM_NCLBUTTONDOWN:
          if(wParam >= HTLEFT && wParam <= HTBOTTOMRIGHT)
              hOldCursor = SetCursor(hSizeCursor);
          return(DefWindowProc(hwnd, msg, wParam, lParam));
          break;

      case WM_LBUTTONDBLCLK:
      case WM_NCLBUTTONDBLCLK:
           if(wParam != HTCAPTION && wParam != HTSYSMENU)
           {
               GetCursorPos(&pt);
               hPup = GetSubMenu(GetMenu(hwnd), 0);
               TrackPopupMenu(hPup, 0, pt.x, pt.y, 0, hwnd, NULL);
           }
           return(DefWindowProc(hwnd, msg, wParam, lParam));

      case WM_PAINT:
          hdc = BeginPaint(hwnd, &ps);
          if(ShowBitmap)
              DisplayBMback(hwnd, hdc, "AppBitmap", TRUE);
          else
              PaintWNDhatch(hwnd, hdc);
          EndPaint(hwnd, &ps);
          return(TRUE);

      case WM_SYSCOMMAND:
           switch (wParam & 0xFFF0)
             {
               case SC_CLOSE:
                    if(IDOK==(MessageBox(hwnd, "Exit... Are You Sure?",
                              " EXIT APPLICATION ",
                              MB_ICONQUESTION | MB_OKCANCEL)))
                      {
                        KillTimer(hwndClockBar, 1);
                        DeleteDC(hdcMem);
                        return(DefWindowProc(hwnd, msg, wParam, lParam));
                      }
                    return(TRUE);

               case SC_MOUSEMENU: /* System menu box */
                    strcpy(szMsg, "System Menu Selections...");
                    return(DefWindowProc(hwnd, msg, wParam, lParam));

               default:
                    return(DefWindowProc(hwnd, msg, wParam, lParam));
             }
           return(DefWindowProc(hwnd, msg, wParam, lParam));


      case WM_MENUSELECT:
           hMenu = GetMenu(hwnd);
           hFile = GetSubMenu(hMenu, 0);
           hEdit = GetSubMenu(hMenu, 1);
           hPup = GetSubMenu(hMenu, 2);
           hHelp = GetSubMenu(hMenu, 3);

           if(HIWORD(lParam)==0)
             strcpy(szMsg, szBlank);

           else if((LOWORD(lParam) & MF_POPUP))
             {
               if(wParam==hFile)
                 strcpy(szMsg, "File Operations...");
               else if(wParam==hEdit)
                 strcpy(szMsg, "Editing Operations...");
               else if(wParam==hPup)
                 strcpy(szMsg, "System settings and defaults...");
               else if(wParam==hHelp)
                 strcpy(szMsg, "Help Options...");
             }
           else switch (wParam)
             {
               case IDM_OWNERDRAW:
                    strcpy(szMsg, "Owner Draw Button Example.");
                    break;

               case IDM_BITMAP:
                    strcpy(szMsg, "Toggle background bitmap.");
                    break;

               case IDM_ABOUT:
                    strcpy(szMsg, "Basic information about program.");
                    break;

               case IDM_EXIT:
                    strcpy(szMsg, "Close open sessions and exit.");
                    break;

               case IDM_UNDO:
                    strcpy(szMsg, "Undo last edit changes.");
                    break;

               case IDM_CUT:
                    strcpy(szMsg, "Cut selection to ClipBoard.");
                    break;

               case IDM_COPY:
                    strcpy(szMsg, "Copy selection to ClipBoard.");
                    break;

               case IDM_PASTE:
                    strcpy(szMsg, "Paste from ClipBoard.");
                    break;

               case IDM_SELECT:
                    strcpy(szMsg, "Select an area for editing.");
                    break;

               case IDM_SELECTALL:
                    strcpy(szMsg, "Select all for editing.");
                    break;

               case IDM_MODE:
                    strcpy(szMsg, "Set operating mode for program.");
                    break;

               case IDM_DISPLAY:
                    strcpy(szMsg, "Set display options for program.");
                    break;

               case IDM_CONFIG:
                    strcpy(szMsg, "Set default configuration.");
                    break;

               case IDM_INDEX:
                    strcpy(szMsg, "Index of general program help.");
                    break;

               case IDM_TOPICS:
                    strcpy(szMsg, "Index of program help topics.");
                    break;

               case IDM_FIND:
                    strcpy(szMsg, "Find match on entered text.");
                    break;

               case IDM_FINDLAST:
                    strcpy(szMsg, "Find next match.");
                    break;

               case IDM_HELPHELP:
                    strcpy(szMsg, "Help on Help.");
                    break;

               case SC_RESTORE:
                    strcpy(szMsg, "Restore program to original size.");
                    break;

               case SC_MOVE:
                    strcpy(szMsg, "Move program window with cursor keys.");
                    break;

               case SC_SIZE:
                    strcpy(szMsg, "Size program window with cursor keys.");
                    break;

               case SC_ICON:
                    strcpy(szMsg, "Iconify program window.");
                    break;

               case SC_ZOOM:
                    strcpy(szMsg, "Maximize program window.");
                    break;

               case SC_CLOSE:
                    strcpy(szMsg, "Close open sessions and exit.");
                    break;

               case 0xF130: /* Task manager */
                    strcpy(szMsg, "Access the Task Manager.");
                    break;

               case 0x1026: /* System menu box */
                    strcpy(szMsg, "System Menu Selections.");
                    break;

               default:
                    strcpy(szMsg, szBlank);
                    break;
             }
           if(NULL != strcmp(szOldMsg, szMsg))
             {
               InvalidateRect(hwndMsgBar, NULL, TRUE);
               UpdateWindow(hwndMsgBar);
             }
           return(DefWindowProc(hwnd, msg, wParam, lParam));


      case WM_COMMAND:
          if(wParam >= IDM_UNDO && wParam <= IDM_HELPHELP)
          {
              MessageBox(hwnd, "Selection not implemented ",
                               "Unimplemented Error",
                               MB_ICONEXCLAMATION | MB_OK);
              break;
           }

           switch (wParam)
             {
               case IDM_BITMAP:
                    hMenu = GetMenu(hwnd);
                    if(ShowBitmap)
                    {
                        ShowBitmap = FALSE;
                        CheckMenuItem(hMenu,IDM_BITMAP, MF_UNCHECKED);
                    }
                    else
                    {
                        ShowBitmap = TRUE;
                        CheckMenuItem(hMenu,IDM_BITMAP, MF_CHECKED);
                    }
                    InvalidateRect(hwndClient, NULL, TRUE);
                    UpdateWindow(hwndClient);
                    break;

               case IDM_OWNERDRAW:
                   CallDBcode(hwnd, OwnerDrawDB, "OwnerDrawBox");
                   return(NULL);

               case IDM_WORKSHEET:
                    CallDBcode(hwnd, RectBoxDB, "RectBox");
                    return(NULL);

               case IDM_ABOUT:
                    CallDBcode(hwnd, AboutBoxDB2, "AboutBox");
                    return(NULL);

               case IDM_EXIT:
                    if(IDOK==(MessageBox(hwnd,
                                         "Exit...Are You Sure?",
                                         "Exit Application",
                                          MB_ICONQUESTION | MB_OKCANCEL)))
                      {
                        KillTimer(hwnd, 1);
                        DeleteDC(hdcMem);
                        PostQuitMessage(0);
                      }
                    break;

               default:
                    return(FALSE);
             }
           break;

      default:
           return(DefWindowProc(hwnd, msg, wParam, lParam));
    }
  return(TRUE);
}

long FAR PASCAL RectWndProc(hwnd, msg, wParam, lParam)
HWND hwnd;                  /* FILENAME: RWNDPROC.C */
unsigned msg;
WORD wParam;
LONG lParam;
{
    char Caption[25];
    HBRUSH hBrush;
    HDC hdc;
    PAINTSTRUCT ps;
    RECT rc;

    switch(msg)
    {
        case WM_PAINT:
            GetClientRect(hwnd, &rc);
            GetWindowText(hwnd, Caption, sizeof(Caption));
            hdc = BeginPaint(hwnd, &ps);

            hBrush=CreateSolidBrush(RGB(255, 0, 0));
            hBrush = SelectObject(hdc, hBrush);
            SetBkColor(hdc, RGB(255, 0, 0));

            Rectangle(hdc, rc.left, rc.top,
                        rc.right, rc.bottom);

            DrawText(hdc, Caption, -1, &rc,
                       DT_SINGLELINE | DT_CENTER | DT_VCENTER);

            DeleteObject(SelectObject(hdc, hBrush));
            EndPaint(hwnd, &ps);
            break;

        case WM_KEYUP:
            if(wParam != VK_SPACE)
                break;

        case WM_LBUTTONUP:
            SendMessage(GetParent(hwnd), WM_COMMAND,
                GetWindowWord(hwnd, GWW_ID), (LONG)hwnd);
            break;

        default:
            return(DefWindowProc(hwnd, msg, wParam, lParam));
    }
    return(0L);
}

BOOL FAR PASCAL RectBoxDB(hwndDlg, msg, wParam, lParam)
HWND hwndDlg;
unsigned msg;
WORD wParam;
LONG lParam;
{

    switch(msg)
    {
        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hwndDlg, TRUE);
                    return(TRUE);

                default:
                    break;
            }
            break;
    }
    return(FALSE);
}


BOOL FAR PASCAL AboutBoxDB2(hDlg, msg, wParam, lParam)
HWND     hDlg;
unsigned msg;
WORD     wParam;
LONG     lParam;
{
  if(msg == WM_COMMAND)
    {
      switch (wParam)
        {
          case IDOK:
               EndDialog(hDlg, TRUE);
               break;

          case IDD_ICON:
               MessageBox(hDlg, "Selection not implemented",
                                "Unimplemented Error",
                                 MB_ICONEXCLAMATION | MB_OK);
               EndDialog(hDlg, TRUE);
               break;
        }
      return(TRUE);
    }
  else return(FALSE);
}

BOOL FAR PASCAL OwnerDrawDB(hDlg, msg, wParam, lParam)
HWND hDlg;
unsigned msg;
WORD wParam;
LONG lParam;
{
    static BOOL bLastState;
    HMENU hMenu;
    BOOL DrawButton;
    HWND hCtl;
    LPDRAWITEMSTRUCT lpds;

    switch (msg)
    {
      case WM_CREATE:
           DrawButton = TRUE;
           hCtl = GetDlgItem(hDlg,  IDD_OWNDRW);
           return(TRUE);

      case WM_DRAWITEM:
          if(DrawButton)
          {
              hMenu = GetMenu(hwndClient);
              if(ShowBitmap)
              {
                  DrawBMbutton(hDlg, "OnBitmap", (LPDRAWITEMSTRUCT)lParam);
                  SetDlgItemText(hDlg, IDD_STATE, "Bitmap On");
                  CheckMenuItem(hMenu,IDM_BITMAP, MF_CHECKED);
                  bLastState = FALSE;
              }
              else
              {
                  DrawBMbutton(hDlg, "OffBitmap", (LPDRAWITEMSTRUCT)lParam);
                  SetDlgItemText(hDlg, IDD_STATE, "Bitmap Off");
                  CheckMenuItem(hMenu,IDM_BITMAP, MF_UNCHECKED);
                  bLastState = TRUE;
              }
              DrawButton = FALSE;
          }

          return(TRUE);
          break;

      case WM_COMMAND:
           switch (wParam)
             {
               case IDD_OWNDRW:
                    ShowBitmap = bLastState;
                    MessageBeep(hDlg);
                    DrawButton = TRUE;
                    InvalidateRect(GetDlgItem(hDlg, wParam), NULL, TRUE);
                    UpdateWindow(GetDlgItem(hDlg, wParam));
                    return(TRUE);
                    break;

               case IDOK:
                    EndDialog (hDlg, NULL);
                    return(TRUE);
                    break;

               default:
                    return(FALSE);
                    break;
             }
           return(FALSE);
           break;

      default:
           return(FALSE);
           break;
    }
    return(FALSE);
}


long FAR PASCAL MessageBarWndProc(hwnd, msg, wParam, lParam)
HWND hwnd;                         /* FILENAME: MSGPROC.C */
unsigned msg;
WORD wParam;
LONG lParam;
{
    PAINTSTRUCT ps;
    HBRUSH hBrush, hOldBrush;
    RECT rc, rcMsgBar;

    switch (msg)
    {
      case WM_CREATE:
           if(hOldCursor != hAppCursor)
             hOldCursor = SetCursor(hAppCursor);
           strset(szBlank, 32);
           return(TRUE);
           break;

      case WM_PAINT:
          BeginPaint(hwndMsgBar, &ps);
          SetBkColor(ps.hdc, RGB(255, 0, 0));
          SetTextColor(ps.hdc, RGB(255, 255, 255));
          GetClientRect(hwndClient, &rc);
          SetWindowPos(hwndMsgBar, NULL,
                          rc.left,
                          rc.bottom-GetSystemMetrics(SM_CYMENU),
                          rc.left+rc.right-71,
                          GetSystemMetrics(SM_CYMENU),
                          SWP_NOZORDER);
          GetClientRect(hwndMsgBar, &rcMsgBar);
          hBrush=CreateSolidBrush(RGB(255, 0, 0));
          hOldBrush=SelectObject(ps.hdc, hBrush);
          FillRect(ps.hdc, &rcMsgBar, hBrush);
          SetRect(&rcMsgBar,
                     rcMsgBar.left+4,
                     rcMsgBar.top,
                     rcMsgBar.right,
                     rcMsgBar.bottom);
          DrawText(ps.hdc, (LPSTR)szMsg, -1, &rcMsgBar,
                      DT_SINGLELINE | DT_LEFT);
          SelectObject(ps.hdc, hOldBrush);
          DeleteObject(hBrush);
          strcpy(szOldMsg, szMsg);
          EndPaint(hwndMsgBar, &ps);
          return(TRUE);
          break;

      default:
           return(DefWindowProc(hwnd, msg, wParam, lParam));
           break;
    }
  return(0L);
}



long FAR PASCAL ClockBarWndProc(hwnd, msg, wParam, lParam)
HWND     hwnd;                  /* FILENAME: CLCKPROC.C */
unsigned msg;
WORD     wParam;
LONG     lParam;
{
    PAINTSTRUCT ps;
    HBRUSH hOldBrush;
    RECT rc, rcClockBar;

    switch (msg)
    {
       case WM_CREATE:
           if(hOldCursor != hAppCursor)
             hOldCursor = SetCursor(hAppCursor);
           strcpy(szCurTime, "HH:MM:SS");
           return(DefWindowProc(hwnd, msg, wParam, lParam));
           break;

       case WM_PAINT:
           BeginPaint(hwnd, &ps);
           SetBkColor(ps.hdc, RGB(255, 255, 255));
           SetTextColor(ps.hdc, RGB(0, 0, 0));
           GetClientRect(hwndClient, &rc);
           MoveWindow(hwnd,
                        rc.right - 71,
                        rc.bottom - GetSystemMetrics(SM_CYMENU),
                        71,
                        GetSystemMetrics(SM_CYMENU),
                        TRUE);
           hOldBrush=SelectObject(ps.hdc, GetStockObject(WHITE_BRUSH));
           GetClientRect(hwndClockBar, &rcClockBar);
           DrawText(ps.hdc, (LPSTR)szCurTime, -1, &rcClockBar,
                      DT_SINGLELINE | DT_CENTER);
           SelectObject(ps.hdc, hOldBrush);
           EndPaint(hwnd, &ps);
           return(TRUE);
           break;

      case WM_TIMER:
           if(wParam==1)
             {
               struct dostime_t Time;
               char             Hr[3], Mn[3], Se[3];

               _dos_gettime(&Time);
               if(Time.hour<10)
                 wsprintf((LPSTR)Hr, "0%d", Time.hour);
               else
                 wsprintf((LPSTR)Hr, "%d", Time.hour);
               if(Time.minute<10)
                 wsprintf((LPSTR)Mn, "0%d", Time.minute);
               else
                 wsprintf((LPSTR)Mn, "%d", Time.minute);
               if(Time.second<10)
                 wsprintf((LPSTR)Se, "0%d", Time.second);
               else
                 wsprintf((LPSTR)Se, "%d", Time.second);
               wsprintf((LPSTR)szCurTime, (LPSTR)"%s:%s:%s",
                        (LPSTR)Hr, (LPSTR)Mn, (LPSTR)Se);

               InvalidateRect(hwndClockBar, NULL, TRUE);
               UpdateWindow(hwndClockBar);
                            return(TRUE);
               break;
             }
      default:
           return(DefWindowProc(hwnd, msg, wParam, lParam));
           break;
    }
  return(0L);
}

void DisplayBMback(hwnd, hdc, lpBitmap, bStretch) /* FILENAME: BMBACK.C */
HWND hwnd;
HDC hdc;
LPSTR lpBitmap;
BOOL bStretch;
{
    RECT rc;
    HANDLE hOldObject, hBitmap;
    int Offset, bmX, bmY, bmWide, bmHigh;

    GetClientRect(hwnd, &rc);

    Offset = GetSystemMetrics(SM_CYMENU)+1;
    bmX = rc.left;
    bmY = rc.top;
    bmWide = rc.right - rc.left;

    if(!hdcMem)
        hdcMem = CreateCompatibleDC(hdc);

    hBitmap = LoadBitmap(hInst, lpBitmap);
    hOldObject = SelectObject(hdcMem, hBitmap);

    if(bStretch)
    {
        bmHigh = rc.bottom - rc.top-Offset;
        StretchBlt(hdc, bmX, bmY, bmWide, bmHigh,
                       hdcMem, 0, 0, 320, 200, SRCCOPY);
    }
    else
    {
        bmHigh = rc.bottom - rc.top;
        BitBlt(hdc, bmX, bmY, bmWide, bmHigh,
                              hdcMem, 0, 0, SRCCOPY);
    }

    SelectObject(hdc, hBitmap);
}

BOOL DrawBMbutton(hwnd, lpName, lpds)    /* FILENAME: DRWBMBUT.C */
HWND hwnd;
LPSTR lpName;
LPDRAWITEMSTRUCT lpds;
{

    if(lpds->itemAction == ODA_DRAWENTIRE)
    {
        DisplayBMback(hwnd, lpds->hDC, lpName, FALSE);
        return(TRUE);
    }
    else
        return(FALSE);

}

void PaintWNDhatch(hwnd, hdc)     /* FILENAME: PNTHATCH.C */
HWND hwnd;
HDC hdc;
{
    RECT rc;
    HBRUSH hBrush, hOldBrush;

    SetBkColor(hdc, RGB(255, 255, 255));
    GetClientRect(hwnd, &rc);

    hBrush=CreateHatchBrush(HS_DIAGCROSS, RGB(255, 0, 0));
    hOldBrush=SelectObject(hdc, hBrush);

    FillRect(hdc, &rc, hBrush);
    SelectObject(hdc, hOldBrush);
    DeleteObject(hBrush);
}

/************************************************************************
 *                                                                      *
 *                    Routines from Other Chapters                      *
 *                                                                      *
 ************************************************************************/

int CallDBcode(hwnd, lpProcAddress, lpName)  /* FILENAME: CALLDB.C */
HWND hwnd;
FARPROC lpProcAddress;
LPSTR lpName;
{
    int nResult;
    FARPROC lpProcBox;

    lpProcBox = MakeProcInstance(lpProcAddress, hInst);
    nResult = DialogBox(hInst, lpName, hwnd, lpProcBox);
    FreeProcInstance(lpProcBox);
    return(nResult);
}
